<?php

/****************************
* Main configuration site
*****************************/

require_once('../includes/functions.php');
require_once('../includes/includes.php');

if(count($_POST) == 0 && getActualConfigName() == "") {
	header("Location: setup.php");
}

require_once('../includes/header.php');

/*****************************
* Node-remove button clicked
*****************************/
if(isset($_POST['remove']) && isset($_POST['removenode'])) {
	removeNodeFromNodesList($_POST['removenode']);
	$messagetype = "info";
	$message = "Node ".$_POST['removenode']." removed from List";
	showMessage($messagetype, $message);
	showConfigurationForm("new", $_POST['enabledglobalconfigs']);
	echo "<br /><br />\n";
}

if(isset($_POST['generateIPv6Prefix'])) {
	showConfigurationForm("new");
}

/*****************************
* Save button clicked
*****************************/
if (isset($_POST['save'])) {
	$configname = $_POST['configname'];
	$actualconfig = getActualConfigName();
	$error = checkForm();
	if($error != "") {
		$messagetype = "error";
		$message = $error;
		showMessage($messagetype, $message);
		showConfigurationForm("new");
	} else {
		if($configname != $actualconfig) {
			createNewConfig($configname);
			$actualconfig = getActualConfigName();
		}
		if($configname != $actualconfig) {
			$messagetype = "error";
			$message = "Configname (".$_POST['configname'].") does not match current Configuration ($actualconfig)";
			showMessage($messagetype, $message);
		} else {
			deleteRemovedNodes();
			if(!writePASSWDFile()) {
				$messagetype = "error";
				$message = "Could not write PASSWD-File";
				showMessage($messagetype, $message);
				exit;
			}
			writeNodeFiles();
			unset($_SESSION['addedNodes']);
			unset($_SESSION['prefixes']);
			unset($_SESSION['removedNodes']);
			$messagetype = "success";
			$message = "Saved Configuration '".$_POST['configname']."'";
			showMessage($messagetype, $message);
			showConfigurationForm("edit", 0);
		}	
	}
}

/*****************************
* Adding new nodes to config list
*****************************/
if(isset($_POST['addnodes'])) {
	addNewNodes($_POST['newnodes'], $_POST['prefix'], $_POST['newtype']);
	$messagetype = "info";
	$message = "Added ".$_POST['newnodes']." new ".$_POST['newtype']." node".($_POST['newnodes'] > 1 ? "s" : "")." to the List";
	showMessage($messagetype, $message);
	showConfigurationForm("new", $_POST['enabledglobalconfigs']);
	echo "<br /><br />\n";
}

/*****************************
* Editing a configuration (Edit-Button from setup.php page
*****************************/
if(isset($_POST['edit']) || (count($_POST) == 0 && getActualConfigName() != "")) {
	unset($_SESSION['addedNodes']);
	unset($_SESSION['prefixes']);
	showConfigurationForm("edit", 0);
}

/*****************************
* Create a configuration (Create-Button from setup.php page
*****************************/
if (isset($_POST['create'])) {
	$_POST['configname'] = $_POST['newconfigname'];
	createNewConfig($_POST['configname']);
	unset($_SESSION['addedNodes']);
	unset($_SESSION['prefixes']);
	showConfigurationForm("new", 1);
}

require('../includes/footer.php');

/*****************************
* Check if passwords, prefix and nodes are set
*****************************/
function checkForm() {
	$error = "";
	if($_POST['root_pass'] != $_POST['root_pass1']) {
		$error .= "Root passwords don't match";
	}
	if($_POST['root_pass'] == "" && $_POST['root_pass1'] == "") {
		if($error != "") $error .= ", ";
		$error .= "Insert an Root password";
	}
	if($_POST['ipv6_prefix'] == "") {
		if($error != "") $error .= ", ";
		$error .= "Insert an IPv6 Prefix";
	}
	if(count($_POST['nodes']) <= 0) {
		if($error != "") $error .= ", ";
		$error .= "Insert at least one Node";
	}
	return $error;
}

function writePASSWDFile() {
	global $config;
	$filename = "PASSWD";
	$success = false;
	$configdir = $config['folderTmpNodeConfiguration'];
	$configname = $_POST['configname'];
	if(is_file($configdir.$configname.'/'.$filename)) {
		$success = true;
	} else {
		$success = file_put_contents($configdir . $configname . "/" . $filename  , $_POST['root_pass'] . "\n");
	}
	return $success;
}

/*****************************
* Write .confs and keys for every node
*****************************/
function writeNodeFiles() {
	if(count($_POST['nodes']) > 0) {
		$counter = 0;
		foreach($_POST['nodes'] as $nodename) {
			if(!nodeConfigExists($nodename)) {
				$nodetype = $_POST['nodetype'][$counter];
				$nodeconfiguration = getDefaultNodeConfiguration($nodetype);
				$nodeconfiguration['GLOBAL_CFNET_PREFIX'] = $_POST['ipv6_prefix'];
				writeNodeConfFile($nodename, $nodeconfiguration);
				writeKeys($nodename);
			}
			$counter++;
		}
	}
}

/*****************************
* Generate .priv and .pub key for a node
*****************************/
function writeKeys($nodename) {
	global $config;
	$configdir = $config['folderTmpNodeConfiguration'];
	$configname = $_POST['configname'];
	$filename  = $nodename . ".conf";
	$cfkeycommand = "sudo /sbin/cfkey " . $configdir . $configname . " " . $nodename;
	shell_exec("export HOME=/var/www; $cfkeycommand");
	$chmodcommand1 = "sudo /bin/chmod a+r " . $configdir . $configname . "/root-" . $nodename . ".pub";
        $chmodcommand2 = "sudo /bin/chmod a+r " . $configdir . $configname . "/root-" . $nodename . ".priv";
	shell_exec($chmodcommand1);
        shell_exec($chmodcommand2);
}

/*****************************
* Show the main configuration form
*****************************/
function showConfigurationForm($mode = "new", $enabledglobalconfigs = true) {
	global $config;
	$configdir = $config['folderTmpNodeConfiguration'];
	$configname = $_POST['configname'];
	
	//new configuration
	if($mode == "new") {
		$configname = $_POST['configname'];
		$root_pass = $_POST['root_pass'];
		$root_pass1 = $_POST['root_pass1'];
		$ipv6_prefix = $_POST['ipv6_prefix'];
		if(isset($_POST['generateIPv6Prefix'])) {
			$ipv6_prefix = generateIPv6Prefix();
		}
		echo "<h1>Create new configuration '".$configname."'</h1>\n";
	//existing configuration
	} elseif($mode == "edit") {
		$configname = $_POST['configname'];
		if($configname == "") $configname = getActualConfigName();
		$configuration = readConfiguration($configname);
		if($configuration == "") {
			$messagetype = "error";
			$message = "The configuration '".$_POST['configname']."' does not exist\n";
			showMessage($messagetype, $message);
			exit();
		}

		storeConfiguredNodesInSession($configuration['hosts']);
		$root_pass = $configuration['root_pass'];
		$root_pass1 = $configuration['root_pass1'];
		$ipv6_prefix = $configuration['ipv6_prefix'];
		echo "<h1>Edit configuration '".$configname."'</h1>\n";
	}

	echo "<h2>Network</h2>";	
	echo "<form name='configlist' action='".$_SERVER['PHP_SELF']."' enctype='multipart/form-data' method='post' >\n";
		echo "<input type='hidden' name='configname' value='".$configname."' />\n";
		echo "<input type='hidden' name='removenode' value='' />\n";
		echo "<input type='hidden' name='editnode' value='' />\n";
		echo "<input type='hidden' name='enabledglobalconfigs' value='".$enabledglobalconfigs."' />\n";
		echo "<table class='configuration'>\n";
			echo "<tr>\n";
				echo "<td class='column1'>\n";
					echo "Root password:";
				echo "</td>\n";
				echo "<td class='column2'>\n";
					echo "<input type='password' name='root_pass' value='".$root_pass."' ";
						if(existingPASSWDFile($configdir.$configname)) {
							echo "readonly='readonly' class='disabled' ";
						}
					echo "/>\n";
				echo "</td>\n";
				echo "<td class='column3'>\n";
					echo "Verify:\n";
				echo "</td>\n";
				echo "<td class='column4'>\n";
					echo "<input type='password' name='root_pass1' value='".$root_pass1."' ";
						if(existingPASSWDFile($configdir.$configname)) {
							echo "readonly='readonly' class='disabled' ";
						}
					echo "/>\n";
				echo "</td>\n";
			echo "</tr>\n";
			echo "<tr>\n";
				echo "<td class='column1'>\n";
					echo "IPv6 Prefix:";
				echo "</td>\n";
				echo "<td class='column2'>\n";
					echo "<input type='text' name='ipv6_prefix' value='".$ipv6_prefix."' ";
						if($enabledglobalconfigs == 0) {
							echo "readonly='readonly' class='disabled' ";
						}
					echo "/>\n";
				echo "</td>\n";
				echo "<td class='column3' colspan='2'>\n";
					if($enabledglobalconfigs == 1) {
						echo "<input type='submit' name='generateIPv6Prefix' value='Generate IPv6 Prefix...'/>\n";
					}
				echo "</td>\n";
			echo "</tr>\n";
		echo "</table>\n";
		echo "<br /><br />\n";
		
		showNodesForm();
		showNodesList($configuration['hosts']);

		echo "<input type='submit' name='save' value='Save' class='button'/>\n";
		echo "<input type='reset' name='cancel' value='Cancel' onClick='location.href=\"setup.php\"' class='button'/>\n";

	echo "</form>\n";
}

/*****************************
* Form to add multiple new nodes
*****************************/
function showNodesForm() {
	if(!isset($_POST['prefix']) || $_POST['prefix'] == "")
		$_POST['prefix'] = "Node";
	echo "<h2>Nodes</h2>\n";
	echo "<table class='confignodesform'>\n";
		echo "<tr>\n";
			echo "<td class='column1'>\n";
				echo "Add new Nodes:\n";
			echo "</td>\n";
			echo "<td class='column2'>\n";
				echo "Number:\n";
			echo "</td>\n";
			echo "<td class='column3'>\n";
				echo generateNrDropdown(1,10, "newnodes", $_POST['type']);
			echo "</td>\n";
			echo "<td class='column4'>\n";
				echo "Type:\n";
			echo "</td>\n";
			echo "<td class='column5'>\n";
				$availablenodetypes = getAvailableNodeTypes();
				echo generateTypeDropdown($availablenodetypes, $_POST['newtype'], 'newtype');
			echo "</td>\n";
			echo "<td class='column6'>\n";
				echo "Name (prefix):\n";
			echo "</td>\n";
			echo "<td class='column7'>\n";
				echo "<input type='text' name='prefix' value='".$_POST['prefix']."' />\n";
			echo "</td>\n";
			echo "<td class='column8'>\n";
				echo "<input type='submit' name='addnodes' value='Add' class='button'/>\n";
			echo "</td>\n";
		echo "</tr>\n";
	echo "</table>\n";
	echo "<br /><br />\n";

}

/*****************************
* Existing node list
*****************************/
function showNodesList($configuredHosts) {
	echo "<table class='confignodeslist'>\n";
		echo createNodesListHeader();
		showSessionNodesList();
			
	echo "</table>\n";

	echo "<br /><br />\n";
}

function showExistingNodesList($configuredHosts) {
	if(count($configuredHosts) > 0) {
		foreach($configuredHosts as $nodename => $nodearray) {
			echo createNodesListNode($nodename, $nodearray['GLOBAL_SUBSYSTYPE'], true, true);
		}
	}
}

function showNewNodesList() {
	$nodes = getNodesFromSession();
	foreach($nodes as $node) {
		echo createNodesListNode($node["name"], $node['type'], true, false);
	}	
}

function storeConfiguredNodesInSession($hosts) {
	$nodes = getNodesFromSession();
	foreach($hosts as $nodename => $nodearray) {
		$nodes[] = array("name" => $nodename,
			"type" => $nodearray['GLOBAL_SUBSYSTYPE'],
			"existing" => 1);
		$pattern = '/([a-zA-Z]*)([0-9]*)/';
		if(preg_match($pattern, $nodename, $matches) > 0) {
			$prefix = $matches[1];
			$nbr = $matches[2];
			getNextNodeNumber($prefix, $nbr);
		}
		
	}
	storeNodesInSession($nodes);
}

/*****************************
* Managing the continuous node naming
*****************************/
function getNextNodeNumber($prefix, $number="") {
	if(isset($_SESSION['prefixes'])) {
		$prefixes = $_SESSION['prefixes'];
		if(array_key_exists($prefix, $prefixes)) {
			$nextNodeNumber = $prefixes[$prefix];
			if($number != "" && $number > $nextNodeNumber) {
				$nextNodeNumber = $number;
			}
			$prefixes[$prefix] = $nextNodeNumber+1;
		} else {
			$nextNodeNumber = 0;
			$prefixes[$prefix] = 1;
		}
	} else {
		$prefixes = array();
		$nextNodeNumber = 0;
		$prefixes[$prefix] = 1;
	}
	$_SESSION['prefixes'] = $prefixes;
	return $nextNodeNumber;
}

function showSessionNodesList() {
	$nodes = getNodesFromSession();
	foreach($nodes as $node) {
		echo createNodesListNode($node["name"], $node['type'], true, $node['existing']);
	}
}

function addNewNodes($number, $prefix, $newtype) {
	$addedNodes = getNodesFromSession();
	if(isset($number) && $number > 0) {
		for($i=0; $i<$number; $i++) {
			$addedNodes = addNewNodeToNodesList($addedNodes, $prefix, $newtype, 0);
		}
	}
	storeNodesInSession($addedNodes);
}

function getNodesFromSession() {
	if(isset($_SESSION['addedNodes'])) {
		$addedNodes = $_SESSION['addedNodes'];
	} else {
		$addedNodes = array();
		storeNodesInSession($addedNodes);
	}
	return $addedNodes;
}

function storeNodesInSession($nodes) {
	$_SESSION['addedNodes'] = $nodes;
}

function addNewNodeToNodesList($addedNodes, $prefix, $type, $existing = 0) {
	$newList = $addedNodes;
	$nextNumber = extendDigits(getNextNodeNumber($prefix), 2);	
	$newList[] = array("name" => $prefix . $nextNumber,
			"type" => $type,
			"existing" => $existing);
	return $newList;
}

function removeNodeFromNodesList($nodename) {
	$currentList = getNodesFromSession();
	$removedNodes = array();
	$newList = array();
	foreach($currentList as $node) {
		if($node['name'] != $nodename)	{
			$newList[] = $node;
		} else {
			$removedNodes[] = $node;
		}
	}
	storeNodesInSession($newList);
	$_SESSION['removedNodes'] = $removedNodes;
	//deleteNodeConfig($nodename);
}
function deleteRemovedNodes() {
	$removedNodes = $_SESSION['removedNodes'];
	if(count($removedNodes) > 0) {
		foreach($removedNodes as $removedNode) {
			deleteNodeConfig($removedNode);
		}
		unset($_SESSION['removedNodes']);
	}	
}

/*****************************
* Deleting the .conf file of a node
*****************************/
function deleteNodeConfig($removedNode) {
	global $config;
	$configdir = $config['folderTmpNodeConfiguration'];
	$configname = $_POST['configname'];
	$nodename = $removedNode['name'];
	if(is_file($configdir.$configname.'/'.$nodename.".conf")) {
		unlink($configdir.$configname . "/" . $nodename.".conf");
	}
}

function extendDigits($digit, $numberDigits) {
	$newdigit = "";
	$length = strlen($digit);
	$difference = $numberDigits - $length;
	if($difference < 0) {
		return $digit;
	}
	for($i=0; $i<$difference; $i++) {
		$newdigit .= '0';
	}
	$newdigit .= $digit;
	return $newdigit;
}

function createNodesListHeader() {
	$string = "";
	$string .= "<tr>\n";
		$string .= "<th class='column1'>\n";
			$string .= "Name\n";
		$string .= "</th>\n";
		$string .= "<th class='column2'>\n";
			$string .= "Type\n";
		$string .= "</th>\n";
		$string .= "<th class='column4'>\n";
			$string .= "";
		$string .= "</th>\n";
	$string .= "</tr>\n";
	return $string;
}

function createNodesListNode($name, $type, $pingOk, $existing = false) {
	$string = "";
	$string .= "<tr>\n";
		$string .= "<td class='column1'>\n";
			$string .= "<input type='text' name='nodes[]' value='".$name."'";
			if($existing) $string .= " readonly='readonly' class='disabled'";
			$string .= "/>\n";
		$string .= "</td>\n";
		$string .= "<td class='column2'>\n";
			$availablenodetypes = getAvailableNodeTypes();
			$string .= generateTypeDropdown($availablenodetypes, $type, 'nodetype[]', $existing ? true : false);
		$string .= "</td>\n";
		$string .= "<td class='column4' style='width: 300px;'>\n";
			if($existing) $string .= "<input type='submit' name='edit' value='Edit...' onClick='setEditNode(\"".$name."\");setAction(\"configlist\",\"editnode.php\");' class='button'/>\n";
			$string .= "<input type='submit' name='remove' value='Remove' onClick='setRemoveNode(\"".$name."\");' class='button'/>\n";
		$string .= "</td>\n";
	$string .= "</tr>\n";
	return $string;
}

function generateTypeDropdown($types, $selectedValue = "", $selectname, $disabled=false) {
	if($selectedValue == "")
		$selectedValue = $types[0];
	$string = "";
	if($disabled) {
		$string .= "<input type='text' name='".$selectname."' value='".$selectedValue."' readonly='readonly' class='disabled'/>\n";
	} else {
		$string .= "<select name='".$selectname."'";
		$string .= ">\n";
		foreach($types as $type) {
			if ( $type == $selectedValue) {
				$string .= "<option value='$type' selected='selected'>$type</option>\n";
			} else {
				$string .= "<option value='$type'>$type</option>\n";
			}
		}
		$string .= "</select>\n";
	}
	return $string;
}

?>