<?php
/****************************
* Genererates with graphviz the graph of a specific meshnode
* with its neighbors and the connected sensornodes
*****************************/
	require_once("Image/GraphVizNG.php");

	class Graph_Meshnode {
		public $name;
		public $meshnode;
		public $sensornodeGateways = array();
		public $sensornodes = array();
		
		public function __construct($name, $meshnode) {
			if($name == "" || $meshnode == null) {
				throw new Exception('Name and meshnode needed');
			}
			$this->name = $name;
			$this->meshnode = $meshnode;
			$this->sensornodeGateways = $meshnode->getConnectedSensornodeGateways($this->meshnode->name);
			$this->sensornodes = $meshnode->getConnectedSensornodes($this->meshnode->name);
		}
		
		public function drawGraph() {
			global $config;
			$graphFormat = "svg";
			$dotFile = "./graph.dot";
			$graphFile = "./graph." . $graphFormat;
			$fontname = "arial";
			$fontsize = "14";
	
			$graph = new Image_GraphVizNG(false);
			
			//Graph attributes
			$graph->addAttributes(
				array(
					"overlap" => "scale",
					"ratio" => "fill",
					"splines" => "true",
					"model" => "shortpath",
					"size" => "6.20,6.20!",
					"mode" => "KK",
				)
			);
			

			//Meshnode graphviz 2.8-2.4

			//Selected Meshnode
			$graph->addNode(
				$this->meshnode->name,
				array(
						"shape" => "circle",
						"tooltip" => $this->meshnode->name,
						"image" => "../pics/" . $this->meshnode->type . ".png",
						"label" => $this->meshnode->name,
						"labelloc" => "b",
						"width" => "1.75",
						"height" => "1.5",
						"fixedsize" => "true",
						"color" => "#2656AE",
						"style" => "setlinewidth(2)",
						"peripheries" => "1",
						"fontname" => $fontname,
						"fontsize" => $fontsize,
						"URL" => "javascript:updateGraph('" . $this->meshnode->name . "');",
						"target" => "_top"


					
				)
			);
			//meshnodeneighbours
			$neighbours = $this->meshnode->getNeighbours($this->meshnode->name);
			foreach($neighbours as $neighbour) {


				$graph->addNode(
					$neighbour->name,
					array(
						"shape" => "circle",
						"tooltip" => $neighbour->name,
						"image" => "../pics/" . $neighbour->type . ".png",
						"label" => $neighbour->name,
						"labelloc" => "b",
						"width" => "1.75",
						"height" => "1.5",
						"fixedsize" => "true",
						"color" => "green",
						"peripheries" => "0",
						"fontname" => $fontname,
						"fontsize" => $fontsize,
						"URL" => "javascript:updateGraph('" . $neighbour->name . "');",
						"target" => "_top",

					)
				);
				$edge = array($this->meshnode->name => $neighbour->name);
				$graph->addEdge($edge);
			}
				
			//Sensornodes
 			foreach($this->sensornodes as $sensornode) {
				$graph->addNode(
					"sn".$sensornode->name,
					array(
						"shape" => "circle",
						"tooltip" => "sn".$sensornode->name,
						"image" => "../pics/tmote-sky.png",
						"label" => "sn".$sensornode->name,
						"labelloc" => "b",
						"width" => "1.75",
						"height" => "1.5",
						"fixedsize" => "true",
						"peripheries" => "0",
						"fontname" => $fontname,
						"fontsize" => $fontsize,
						"URL" => "javascript:updateGraph('".$this->meshnode->name."', '".$sensornode->name."');",
						"target" => "_top",
						
					)
				);
			}
			//Edges Sensornode -> Meshnode
			$sensornodeGateways = $this->meshnode->getConnectedSensornodeGateways($this->meshnode->name);
			//print_r($connections);
			foreach($sensornodeGateways as $sensornodeGateway) {
				$edge = array("sn".$sensornodeGateway => $this->meshnode->name);
				$graph->addEdge($edge);
			}

			//Edges Sensornodes -> Sensornode
			$sensornodeConnections = $this->meshnode->getSensornodeConnections($this->meshnode->name);
//			print_r($sensornodeConnections);
			foreach($sensornodeConnections as $sensornodeConnection) {
				foreach($sensornodeConnection as $sn_id => $sn_neighbor_id) {
					$edge = array("sn".$sn_id => "sn".$sn_neighbor_id);
					$graph->addEdge($edge);
				}
			}

			//generating image
			$graph->saveParsedGraph($dotFile);
			$graph->renderDotFile($dotFile, $graphFile, $graphFormat, false);
			
			if($graphFormat == 'svg') {
				echo "<object data='" . $graphFile . "?" . filemtime($graphFile) . "' width='620' height='620' type='image/svg+xml'>";
					echo "<param name='src' value='" . $graphFile . "'>";
				echo "</object>";
			} else {
				echo "<img src='" . $graphFile . "'>";
			}
		}
	}
?>
