<?php
/****************************
* Genererates with graphviz the graph of a complete network
* with the connected meshnodes and attached sensor networks
*****************************/
	require_once("Image/GraphVizNG.php");

	class Graph_Network {
		public $name;
		public $network;
		public $meshnodes;
		public $meshnode_positions;
	
		public function __construct($name, $network) {
			if($name == "" || $network == null) {
				throw new Exception('Name and meshnodes needed');
			}
			$this->name = $name;
			$this->network = $network;
			$this->meshnodes = $network->getAllMeshnodes();
			
		}
		
		public function drawGraph() {
			global $config;
			$graphFormat = "svg";
			$dotFile = "./graph.dot";
			$graphFile = "./graph." . $graphFormat;
			$fontname = "arial black";
			$fontsize = "14";
		
			$graph = new Image_GraphVizNG(false);
			
			//Graph attributes
			$graph->addAttributes(
				array(
					"overlap" => "true",
 					"ratio" => "fill",
					"splines" => "false",
					"model" => "shortpath",
					"size" => "11.11,16.69",
					"mode" => "KK",
					"dpi" => "72.0",
					"margin" => "0.0,0.0",
					"mindist" => "0.0",
					"pack" => "false",
				)
			);
			
			//Nodes
			//fakepoints on the edges of the image, because of correct scaling the sensor nodes
			$graph->addNode(
				"f0",
				array(
					"shape" => "box", 
					"color" => 'transparent', 
					"fontcolor" => "transparent", 
					"width" => "0.1", 
					"height" => "0.1", 
					"fixedsize" => "true", 
					"pos" => "0,0", 
					"pin" => "true"));
			$graph->addNode(
				"f1", 
				array(
					"shape" => "box", 
					"color" => 'transparent', 
					"fontcolor" => "transparent", 
					"width" => "0.1", 
					"height" => "0.1", 
					"fixedsize" => "true", 
					"pos" => "800,0", 
					"pin" => "true"));
			$graph->addNode(
				"f2", 
				array(
					"shape" => "box", 
					"color" => 'transparent', 
					"fontcolor" => "transparent", 
					"width" => "0.1", 
					"height" => "0.1", 
					"fixedsize" => "true", 
					"pos" => "800,1202", 
					"pin" => "true"));
			$graph->addNode(
				"f3", 
				array(
					"shape" => "box", 
					"color" => 'transparent', 
					"fontcolor" => "transparent", 
					"width" => "0.1", 
					"height" => "0.1", 
					"fixedsize" => "true", 
					"pos" => "0,1202", 
					"pin" => "true"));


			foreach($this->meshnodes as $meshnode) {
				$node_3d_x_m = $meshnode->x;
				$node_3d_x_px = $this->m2px($node_3d_x_m, $config['building_x_m'], $config['building_x_px']);
				$node_3d_y_m = $meshnode->y;
				$node_3d_y_px = $this->m2px($node_3d_y_m, $config['building_y_m'], $config['building_y_px']);
				$node_3d_z_m = $meshnode->z;
				$node_3d_z_px = $this->m2px($node_3d_z_m, $config['building_z_m'], $config['building_z_px']);

				$node_2d_x_px = round($this->get_2d_x($node_3d_x_px, $node_3d_y_px, $config['graph_alpha']));
				$node_2d_y_px = round($this->get_2d_y($node_3d_y_px, $node_3d_z_px, $config['graph_alpha'], $config['floor_px']));
				
				$this->meshnode_positions[$meshnode->name] = array("x" => $node_2d_x_px, "y" => $node_2d_y_px);

				$graph->addNode(
					$meshnode->name,
					array(
						"shape" => "box",
						"tooltip" => $meshnode->name,
						"image" => "../pics/" . $meshnode->type . ".png",
						"label" => $meshnode->name,
						"labelloc" => "b",
						"width" => "0.5",
						"height" => "0.8",
						"fixedsize" => "true",
						"peripheries" => "0",
						"fontname" => $fontname,
						"fontsize" => $fontsize,
						"URL" => "javascript:updateGraph('" . $meshnode->name . "');",
						"target" => "_top",
						"pos" => $node_2d_x_px . "," . $node_2d_y_px,
						"pin" => "true"
					)
				);

				//Check if meshnode has attached sensornodes. if yes -> draw sensornodenetwork circle
				$snGateways = array();
				$snGateways = $meshnode->getConnectedSensornodeGateways($meshnode->name);
				if(count($snGateways) > 0) {
					$graph->addNode(
					"wsn".$snGateways[0],
					array(
						"shape" => "ellipse",
						"tooltip" => "wsn".$snGateways[0],
						"label" => "wsn" . $snGateways[0],
						"labelloc" => "t",
						"width" => "3",
						"height" => "1.5",
						"fixedsize" => "true",
						"peripheries" => "1",
						"style" => "setlinewidth(2)",
						"fontname" => $fontname,
						"fontsize" => $fontsize,
						"color" => "green",
						"pos" => $node_2d_x_px . "," . $node_2d_y_px
					));
				}

			}

			//Edges Meshnode -> Meshnode
			$connections = $this->network->getAllConnections();
			//print_r($connections);
			foreach($connections as $connection) {
				foreach($connection as $meshnode => $neighbour) {
					$edge = array(
						$meshnode => $neighbour,
						"arrowhead" => "normal",
						"arrowtail" => "normal",
					);
	//				print_r($edge);
					if(!$graph->containsEdge($edge)) {
						$graph->addEdge($edge);
					}
				}
			}

			//generating image
			$graph->saveParsedGraph($dotFile);
			$graph->renderDotFile($dotFile, $graphFile, $graphFormat, true);
			
			//making svg transparent. (removing white polygone in svg-code)
			$svgfile = file_get_contents($graphFile);

			$pattern = "/<polygon.*\/>\n/";
			$replacement = "";
			$newSvgFile = preg_replace($pattern, $replacement, $svgfile);
			
			//saving "new" svg
			file_put_contents($graphFile, $newSvgFile);

			if($graphFormat == 'svg') {
				echo "<object id='networkgraphobj' data='" . $graphFile . "?" . filemtime($graphFile) . "' width='800' height='1202' type='image/svg+xml'>";
					echo "<param name='src' value='" . $graphFile . "'>";
				echo "</object>";
			} else {
				echo "<img src='" . $graphFile . "'>";
			}
		}

		public function m2px($node_m, $building_m, $building_px) {
			return $building_px/$building_m*$node_m;
		}

		public function get_2d_x($x3d, $y3d, $alpha) {
			return $x3d + $y3d*cos(deg2rad($alpha));
		}

		public function get_2d_y($y3d, $z3d, $alpha, $floor) {
			return $y3d*sin(deg2rad($alpha)) + $z3d + $floor;
		}
	}
?>
