<?php

/****************************
* Generating the network.xml for the dhtmlxtree (tree on the right side)
*****************************/

class XMLGenerator {

	private static $instance = null;
	private $doc = null;
	private $network = null;
	
	private function __construct(){}
	
	public function getInstance() {
		if(self::$instance == null ) {
			self::$instance = new XMLGenerator();
		}
		return self::$instance;
	}

	public final function __clone() {
		trigger_error( "Cannot clone instance of XMLGenerator", E_USER_ERROR );
	}

	public function generateNetworkXML($network) {

		$this->network = $network;
		
		$this->doc = new DOMDocument("1.0", "ISO-8859-1");
		
		$tntree = $this->createTree();
		$this->doc->appendChild($tntree);
		
		file_put_contents("../network/network.xml", $this->doc->saveXML());
	}

	private function createTree() {
		$tntree = $this->doc->createElement("tree");
		$tntree->setAttribute( "id", "0" );

		$tnnetwork = $this->createNetwork();
		$tntree->appendChild($tnnetwork);

		return $tntree;
	}

	private function createNetwork() {
		$tnnetwork = $this->doc->createElement("item");
		$tnnetwork->setAttribute("id", "network");
		$tnnetwork->setAttribute("text", "Network");
		$tnnetwork->setAttribute("im0", "mesh/net.png");
		$tnnetwork->setAttribute("im1", "mesh/net.png");
		$tnnetwork->setAttribute("im2", "mesh/net.png");
		$tnnetwork->setAttribute("closeable", "0");
		$tnnetwork->setAttribute("open", "1");

		foreach($this->network->getAllMeshnodes() as $meshnode) {
			$tnmeshnode = $this->createMeshnode($meshnode);
			$tnnetwork->appendChild($tnmeshnode);
		}

		return $tnnetwork;
	}

	private function createMeshnode($meshnode) {

		$tnmeshnode = $this->doc->createElement("item");
		$tnmeshnode->setAttribute("id", "mn_".$meshnode->name);
		$tnmeshnode->setAttribute("text", $meshnode->name);
		$tnmeshnode->setAttribute("im0", "mesh/" . strtolower($meshnode->type) . ".png");
		$tnmeshnode->setAttribute("im1", "mesh/" . strtolower($meshnode->type) . ".png");
		$tnmeshnode->setAttribute("im2", "mesh/" . strtolower($meshnode->type) . ".png");
		
		$tninformations = $this->createMeshnodeInformations($meshnode);
		$tnmeshnode->appendChild($tninformations);

		$tnsensornodes = $this->createSensornodes($meshnode);
		$tnmeshnode->appendChild($tnsensornodes);

		$tnneighbours = $this->createMeshnodeNeighbours($meshnode);
		$tnmeshnode->appendChild($tnneighbours);

		return $tnmeshnode;
	}

	private function createMeshnodeInformations($meshnode) {
		$tninformations = $this->doc->createElement("item");
		$tninformations->setAttribute("id", "informations_mn_".$meshnode->name);
		$tninformations->setAttribute("text", "Informations");
		$tninformations->setAttribute("nocheckbox", "1");
		$tninformations->setAttribute("im0", "mesh/information.png");
		$tninformations->setAttribute("im1", "mesh/information.png");
		$tninformations->setAttribute("im2", "mesh/information.png");

		$informations = $meshnode->getInformations($meshnode->name);
		foreach($informations as $key => $value) {
			$tninformation = $this->createMeshnodeInformation($key, $value, $meshnode);
			$tninformation->appendChild($tninformation);
		}

		return $tninformations;
	}
	
	private function createMeshnodeInformation($key, $value, $meshnode) {
		$tninformation = $this->doc->createElement("item");
		$tninformation->setAttribute("id", "information_mn_".$meshnode->name."_".$key);
		$tninformation->setAttribute("text", "$key: $value");
		$tninformation->setAttribute("im0", "mesh/value.gif");
		$tninformation->setAttribute("im1", "mesh/value.gif");
		$tninformation->setAttribute("im2", "mesh/value.gif");

		return $tninformation;
	}

	private function createSensornodes($meshnode) {
		$tnsensornodes = $this->doc->createElement("item");
		$tnsensornodes->setAttribute("id", "sensornodes_mn_".$meshnode->name);
		$tnsensornodes->setAttribute("text", "Sensornodes");
		$tnsensornodes->setAttribute("im0", "mesh/tmote-sky.png");
		$tnsensornodes->setAttribute("im1", "mesh/tmote-sky.png");
		$tnsensornodes->setAttribute("im2", "mesh/tmote-sky.png");

		$sensornodes = $meshnode->getConnectedSensornodes($meshnode->name);
		foreach($sensornodes as $sensornode) {
			$tnsensornode = $this->createSensornode($sensornode, $meshnode);
			$tnsensornodes->appendChild($tnsensornode);
		}

		return $tnsensornodes;

	}

	private function createSensornode($sensornode, $meshnode) {
		$tnsensornode = $this->doc->createElement("item");
		$tnsensornode->setAttribute("id", "sensornodes_mn_".$meshnode->name."_sn_".$sensornode->name);
		$tnsensornode->setAttribute("text", $sensornode->name);
		$tnsensornode->setAttribute("im0", "mesh/tmote-sky.png");
		$tnsensornode->setAttribute("im1", "mesh/tmote-sky.png");
		$tnsensornode->setAttribute("im2", "mesh/tmote-sky.png");
		
 		$tninformations = $this->createSensornodeInformations($sensornode, $meshnode);
 		$tnsensornode->appendChild($tninformations);
 
 		$tnneighbours = $this->createSensornodeNeighbours($sensornode, $meshnode);
 		$tnsensornode->appendChild($tnneighbours);

		return $tnsensornode;

	}

	private function createSensornodeInformations($sensornode, $meshnode) {
		$tninformations = $this->doc->createElement("item");
		$tninformations->setAttribute("id", "informations_sn_".$sensornode->name);
		$tninformations->setAttribute("text", "Informations");
		$tninformations->setAttribute("nocheckbox", "1");
		$tninformations->setAttribute("im0", "mesh/information.png");
		$tninformations->setAttribute("im1", "mesh/information.png");
		$tninformations->setAttribute("im2", "mesh/information.png");

		$informations = $sensornode->getInformations($meshnode->name);
		foreach($informations as $information) {
			foreach($information as $key => $value) {
				$tninformation = $this->createSensornodeInformation($key, $value, $sensornode);
				$tninformations->appendChild($tninformation);
			}
		}

		return $tninformations;
	}
	
	private function createSensornodeInformation($key, $value, $sensornode) {
		$tninformation = $this->doc->createElement("item");
		$tninformation->setAttribute("id", "information_sn_".$sensornode->name."_".$key);
		$tninformation->setAttribute("text", "$key: $value");
		$tninformation->setAttribute("nocheckbox", "1");
		$tninformation->setAttribute("im0", "mesh/value.gif");
		$tninformation->setAttribute("im1", "mesh/value.gif");
		$tninformation->setAttribute("im2", "mesh/value.gif");

		return $tninformation;
	}

	private function createMeshnodeNeighbours($meshnode) {
		$tnneighbours = $this->doc->createElement("item");
		$tnneighbours->setAttribute("id", "neighbours_mn_".$meshnode->name);
		$tnneighbours->setAttribute("text", "Neighbours");
		$tnneighbours->setAttribute("nocheckbox", "1");
		$tnneighbours->setAttribute("im0", "mesh/neighbours.png");
		$tnneighbours->setAttribute("im1", "mesh/neighbours.png");
		$tnneighbours->setAttribute("im2", "mesh/neighbours.png");

		$neighbours = $meshnode->getNeighbours();
		foreach($neighbours as $neighbour) {
			$tnneighbour = $this->createMeshnodeNeighbour($neighbour, $meshnode);
			$tnneighbours->appendChild($tnneighbour);
		}

		return $tnneighbours;
	}

	private function createMeshnodeNeighbour($neighbour, $meshnode) {
		$tnneighbour = $this->doc->createElement("item");
		$tnneighbour->setAttribute("id", "neighbour_mn_".$meshnode->name."_mn_".$neighbour->name);
		$tnneighbour->setAttribute("text", $neighbour->name);
		$tnneighbour->setAttribute("nocheckbox", "1");
		$tnneighbour->setAttribute("im0", "mesh/" . strtolower($neighbour->type) . ".png");
		$tnneighbour->setAttribute("im1", "mesh/" . strtolower($neighbour->type) . ".png");
		$tnneighbour->setAttribute("im2", "mesh/" . strtolower($neighbour->type) . ".png");
		
		return $tnneighbour;
	}

	private function createSensornodeNeighbours($sensornode, $meshnode) {
		$tnneighbours = $this->doc->createElement("item");
		$tnneighbours->setAttribute("id", "neighbours_sn_".$sensornode->name);
		$tnneighbours->setAttribute("text", "Neighbours");
		$tnneighbours->setAttribute("nocheckbox", "1");
		$tnneighbours->setAttribute("im0", "mesh/neighbours.png");
		$tnneighbours->setAttribute("im1", "mesh/neighbours.png");
		$tnneighbours->setAttribute("im2", "mesh/neighbours.png");

		$neighbours = $sensornode->getNeighbours($meshnode->name);
		foreach($neighbours as $neighbour) {
			$tnneighbour = $this->createSensornodeNeighbour($neighbour, $sensornode);
			$tnneighbours->appendChild($tnneighbour);
		}

		return $tnneighbours;
	}

	private function createSensornodeNeighbour($neighbour, $sensornode) {
		$tnneighbour = $this->doc->createElement("item");
		$tnneighbour->setAttribute("id", "neighbour_sn_".$sensornode->name."_sn_".$neighbour->name);
		$tnneighbour->setAttribute("text", $neighbour->name);
		$tnneighbour->setAttribute("nocheckbox", "1");
		$tnneighbour->setAttribute("im0", "mesh/tmote-sky.png");
		$tnneighbour->setAttribute("im1", "mesh/tmote-sky.png");
		$tnneighbour->setAttribute("im2", "mesh/tmote-sky.png");
		
		return $tnneighbour;
	}

}
?>